using System;
using System.Collections;
using Server;
using Server.Gumps;
using Server.Multis;
using Server.Network;
using Server.ContextMenus;
using Server.Engines.PartySystem;
using Server.GenerateRandomMagicItems;

namespace Server.Items
{
	[FlipableAttribute( 0xE41, 0xE40 )]
	public class TreasureMapChest : LockableContainer
	{
		private static int[] m_Gems = new int[9];
		private static Type[] m_GemTypes = new Type[]
			{
				typeof( Amber ), typeof( Amethyst ),
				typeof( Citrine ), typeof( Diamond ),
				typeof( Emerald ), typeof( Ruby ),
				typeof( Sapphire ), typeof( StarSapphire ),
				typeof( Tourmaline )
			};

		private static int[] m_Regs = new int[8];
		private static Type[] m_RegTypes = new Type[]
			{
				typeof( BlackPearl ), typeof( Bloodmoss ),
				typeof( Garlic ), typeof( Ginseng ),
				typeof( MandrakeRoot ), typeof( Nightshade ),
				typeof( SpidersSilk ), typeof( SulfurousAsh )
			};

		private int m_Level;
		private DateTime m_DeleteTime;
		private Timer m_Timer;
		private Mobile m_Owner;

		[CommandProperty( AccessLevel.GameMaster )]
		public int Level{ get{ return m_Level; } set{ m_Level = value; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Owner{ get{ return m_Owner; } set{ m_Owner = value; } }

		[CommandProperty( AccessLevel.GameMaster )]
		public DateTime DeleteTime{ get{ return m_DeleteTime; } }

		[Constructable]
		public TreasureMapChest( int level ) : this( null, level )
		{
		}

		public TreasureMapChest( Mobile owner, int level ) : base( 0xE41, 0x42, 0x42, new Rectangle2D( 18, 105, 144, 73 ) )
		{
			m_Owner = owner;
			m_Level = level;
			m_DeleteTime = DateTime.Now + TimeSpan.FromHours( 3.0 );

			m_Timer = new DeleteTimer( this, m_DeleteTime );

			Movable = false;
			TrapType = TrapType.ExplosionTrap;
			TrapPower = level * 25;
			Locked = true;

			switch ( level )
			{
				case 1: RequiredSkill = 36; break;
				case 2: RequiredSkill = 76; break;
				case 3: RequiredSkill = 84; break;
				case 4: RequiredSkill = 92; break;
				case 5: RequiredSkill = 100; break;
			}

			LockLevel = this.RequiredSkill - 10;
			MaxLockLevel = this.RequiredSkill + 40;

			DropItem( new Gold( level * 1000 ) );

			for ( int i = 0; i < level * 5; ++i )
				DropItem( RandomScroll.CreateScroll( 1, 8 ) );

			for ( int i = 0; i < level * 6; ++i )
				DropItem( RandomMagicItem.RandomMagicEquipment( 0, 5 ) );

			ClearAmounts( m_Gems );
			ClearAmounts( m_Regs );

			for ( int i = 0; i < level * 10; ++i )
				m_Gems[Utility.Random( m_Gems.Length )]++;

			for ( int i = 0; i < level * 20; ++i )
				m_Regs[Utility.Random( m_Regs.Length )]++;

			AddItems( m_Gems, m_GemTypes );
			AddItems( m_Regs, m_RegTypes );
		}

		private ArrayList m_Lifted = new ArrayList();

		private bool CheckLoot( Mobile m, bool criminalAction )
		{
			if ( m_Owner == null || m == m_Owner )
				return true;

			Party p = Party.Get( m_Owner );

			if ( p != null && p.Contains( m ) )
				return true;

			if ( this.Map == Map.Felucca )
			{
				if ( criminalAction )
					m.CriminalAction( true );
				else
					m.SendLocalizedMessage( 1010630 ); // Taking someone else's treasure is a criminal offense!

				return true;
			}

			m.SendLocalizedMessage( 1010631 ); // You did not discover this chest!
			return false;
		}

		public override bool CheckItemUse( Mobile from, Item item )
		{
			return CheckLoot( from, item != this ) && base.CheckItemUse( from, item );
		}

		public override bool CheckLift( Mobile from, Item item )
		{
			return CheckLoot( from, true ) && base.CheckLift (from, item);
		}

		public override void OnItemLifted( Mobile from, Item item )
		{
			bool notYetLifted = !m_Lifted.Contains( item );

			from.RevealingAction();

			if ( notYetLifted )
			{
				m_Lifted.Add( item );

				if ( 0.1 >= Utility.RandomDouble() ) // 10% chance to spawn a new monster
					TreasureMap.Spawn( m_Level, GetWorldLocation(), Map, from );
			}

			base.OnItemLifted( from, item );
		}

		public override bool OnDragDropInto( Mobile from, Item item, Point3D p )
		{
			from.SendLocalizedMessage( 1048122, "", 0x8A5 ); // The chest refuses to be filled with treasure again.

			return false;
		}

		private void ClearAmounts( int[] list )
		{
			for ( int i = 0; i < list.Length; ++i )
				list[i] = 0;
		}

		private static object[] m_Arguments = new object[1];

		private void AddItems( int[] amounts, Type[] types )
		{
			for ( int i = 0; i < amounts.Length && i < types.Length; ++i )
			{
				if ( amounts[i] > 0 )
				{
					try
					{
						m_Arguments[0] = amounts[i];
						Item item = (Item)Activator.CreateInstance( types[i], m_Arguments );

						DropItem( item );
					}
					catch
					{
					}
				}
			}
		}

		public TreasureMapChest( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 1 ); // version

			writer.Write( m_Owner );

			writer.Write( (int) m_Level );
			writer.Write( m_DeleteTime - DateTime.Now );
			writer.WriteItemList( m_Lifted, true );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 1:
				{
					m_Owner = reader.ReadMobile();

					goto case 0;
				}
				case 0:
				{
					m_Level = reader.ReadInt();
					m_DeleteTime = DateTime.Now + reader.ReadTimeSpan();
					m_Lifted = reader.ReadItemList();

					m_Timer = new DeleteTimer( this, m_DeleteTime );

					break;
				}
			}
		}

		public override void OnAfterDelete()
		{
			if ( m_Timer != null )
				m_Timer.Stop();

			m_Timer = null;

			base.OnAfterDelete();
		}

		public override bool IsAccessibleTo( Mobile m )
		{
			if ( !BaseHouse.CheckAccessible( m, this ) )
				return false;

			return base.IsAccessibleTo( m );
		}

		public override void GetContextMenuEntries(Mobile from, ArrayList list)
		{
			base.GetContextMenuEntries( from, list );

			list.Add( new RemoveEntry( from, this ) );
		}

		public void BeginRemove( Mobile from )
		{
			from.CloseGump( typeof( RemoveGump ) );
			from.SendGump( new RemoveGump( from, this ) );
		}

		public void EndRemove( Mobile from )
		{
			if ( Deleted || from != m_Owner || !from.InRange( GetWorldLocation(), 3 ) )
				return;

			from.SendLocalizedMessage( 1048124, "", 0x8A5 ); // The old, rusted chest crumbles when you hit it.
			this.Delete();
		}

		private class RemoveGump : Gump
		{
			private Mobile m_From;
			private TreasureMapChest m_Chest;

			public RemoveGump( Mobile from, TreasureMapChest chest ) : base( 15, 15 )
			{
				m_From = from;
				m_Chest = chest;

				Closable = false;
				Disposable = false;

				AddPage( 0 );

				AddBackground( 30, 0, 240, 240, 2620 );

				AddHtmlLocalized( 45, 15, 200, 80, 1048125, 0xFFFFFF, false, false ); // When this treasure chest is removed, any items still inside of it will be lost.
				AddHtmlLocalized( 45, 95, 200, 60, 1048126, 0xFFFFFF, false, false ); // Are you certain you're ready to remove this chest?

				AddButton( 40, 153, 4005, 4007, 1, GumpButtonType.Reply, 0 );
				AddHtmlLocalized( 75, 155, 180, 40, 1048127, 0xFFFFFF, false, false ); // Remove the Treasure Chest

				AddButton( 40, 195, 4005, 4007, 2, GumpButtonType.Reply, 0 );
				AddHtmlLocalized( 75, 197, 180, 35, 1006045, 0xFFFFFF, false, false ); // Cancel
			}

			public override void OnResponse( NetState sender, RelayInfo info )
			{
				if ( info.ButtonID == 1 )
					m_Chest.EndRemove( m_From );
			}
		}

		private class RemoveEntry : ContextMenuEntry
		{
			private Mobile m_From;
			private TreasureMapChest m_Chest;

			public RemoveEntry( Mobile from, TreasureMapChest chest ) : base( 6149, 3 )
			{
				m_From = from;
				m_Chest = chest;

				Enabled = ( from == chest.Owner );
			}

			public override void OnClick()
			{
				if ( m_Chest.Deleted || m_From != m_Chest.Owner )
					return;

				m_Chest.BeginRemove( m_From );
			}
		}

		private class DeleteTimer : Timer
		{
			private Item m_Item;

			public DeleteTimer( Item item, DateTime time ) : base( DateTime.Now - time )
			{
				m_Item = item;
				Priority = TimerPriority.OneMinute;
			}

			protected override void OnTick()
			{
				m_Item.Delete();
			}
		}
	}
}