using System;
using System.Collections;
using Server;
using Server.Gumps;
using Server.Network;

namespace Server.SkillHandlers
{
	public class Tracking
	{
		public static void Initialize()
		{
			SkillInfo.Table[(int)SkillName.Tracking].Callback = new SkillUseCallback( OnUse );
		}

		public static TimeSpan OnUse( Mobile m )
		{
			m.SendLocalizedMessage( 1011350 ); // What do you wish to track?

			m.CloseGump( typeof( TrackWhatGump ) );
			m.CloseGump( typeof( TrackWhoGump ) );
			m.SendGump( new TrackWhatGump( m ) );

			return TimeSpan.FromSeconds( 10.0 ); // 10 second delay before beign able to re-use a skill
		}
	}

	public class TrackWhatGump : Gump
	{
		private Mobile m_From;
		private bool m_Success;

		public TrackWhatGump( Mobile from ) : base( 20, 30 )
		{
			m_From = from;
			m_Success = from.CheckSkill( SkillName.Tracking, 0.0, 21.1 );

			AddPage( 0 );

			AddBackground( 0, 0, 440, 135, 5054 );

			AddBackground( 10, 10, 420, 75, 2620 );
			AddBackground( 10, 85, 420, 25, 3000 );

			AddItem( 20, 20, 9682 );
			AddButton( 20, 110, 4005, 4007, 1, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 20, 90, 100, 20, 1018087, false, false ); // Animals

			AddItem( 120, 20, 9607 );
			AddButton( 120, 110, 4005, 4007, 2, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 120, 90, 100, 20, 1018088, false, false ); // Monsters

			AddItem( 220, 20, 8454 );
			AddButton( 220, 110, 4005, 4007, 3, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 220, 90, 100, 20, 1018089, false, false ); // Human NPCs

			AddItem( 320, 20, 8455 );
			AddButton( 320, 110, 4005, 4007, 4, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 320, 90, 100, 20, 1018090, false, false ); // Players
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			if ( info.ButtonID >= 1 && info.ButtonID <= 4 )
				TrackWhoGump.DisplayTo( m_Success, m_From, info.ButtonID - 1 );
		}
	}

	public delegate bool TrackTypeDelegate( Mobile m );

	public class TrackWhoGump : Gump
	{
		private Mobile m_From;
		private int m_Range;

		private static TrackTypeDelegate[] m_Delegates = new TrackTypeDelegate[]
			{
				new TrackTypeDelegate( IsAnimal ),
				new TrackTypeDelegate( IsMonster ),
				new TrackTypeDelegate( IsHumanNPC ),
				new TrackTypeDelegate( IsPlayer )
			};

		private class InternalSorter : IComparer
		{
			private Mobile m_From;

			public InternalSorter( Mobile from )
			{
				m_From = from;
			}

			public int Compare( object x, object y )
			{
				if ( x == null && y == null )
					return 0;
				else if ( x == null )
					return -1;
				else if ( y == null )
					return 1;

				Mobile a = x as Mobile;
				Mobile b = y as Mobile;

				if ( a == null || b == null )
					throw new ArgumentException();

				return m_From.GetDistanceToSqrt( a ).CompareTo( m_From.GetDistanceToSqrt( b ) );
			}
		}

		public static void DisplayTo( bool success, Mobile from, int type )
		{
			if ( !success )
			{
				from.SendLocalizedMessage( 1018092 ); // You see no evidence of those in the area.
				return;
			}

			Map map = from.Map;

			if ( map == null )
				return;

			TrackTypeDelegate check = m_Delegates[type];

			from.CheckSkill( SkillName.Tracking, 21.1, 100.0 ); // Passive gain

			int range = 10 + (int)(from.Skills[SkillName.Tracking].Value / 10);

			ArrayList list = new ArrayList();
			IPooledEnumerable eable = map.GetMobilesInRange( from.Location, range );

			foreach ( Mobile m in eable )
			{
				if ( m != from && (!m.Hidden || m.AccessLevel == AccessLevel.Player || from.AccessLevel > m.AccessLevel) && check( m ) )
					list.Add( m );
			}

			eable.Free();

			if ( list.Count > 0 )
			{
				list.Sort( new InternalSorter( from ) );

				from.SendGump( new TrackWhoGump( from, list, range ) );
				from.SendLocalizedMessage( 1018093 ); // Select the one you would like to track.
			}
			else
			{
				if ( type == 0 )
					from.SendLocalizedMessage( 502991 ); // You see no evidence of animals in the area.
				else if ( type == 1 )
					from.SendLocalizedMessage( 502993 ); // You see no evidence of creatures in the area.
				else
					from.SendLocalizedMessage( 502995 ); // You see no evidence of people in the area.
			}
		}

		private static bool IsAnimal( Mobile m )
		{
			return ( !m.Player && m.Body.IsAnimal );
		}

		private static bool IsMonster( Mobile m )
		{
			return ( !m.Player && m.Body.IsMonster );
		}

		private static bool IsHumanNPC( Mobile m )
		{
			return ( !m.Player && m.Body.IsHuman );
		}

		private static bool IsPlayer( Mobile m )
		{
			return m.Player;
		}

		public static int GetItemID( Mobile m )
		{
			int val = m.Body.BodyID;

			switch ( val )
			{
				case 998: return 0x25A0;
				case 848: return 0x259E;
				case 846: return 0x2599;
				case 843: return 0x2123;
				case 842: return 0x2615;
				case 841: return 0x2615;
				case 840: return 0x2615;
				case 839: return 0x2615;
				case 838: return 0x25C1;
				case 837: return 0x25C0;
				case 835: return 0x2125;
				case 834: return 0x2597;
				case 833: return 0x211A;
				case 832: return 0x211A;
				case 831: return 0x211A;
				case 828: return 0x20F6;
				case 827: return 0x25B4;
				case 826: return 0x25B3;
				case 825: return 0x25B2;
				case 820: return 0x259A;
				case 799: return 0x2619;
				case 796: return 0x2611;
				case 795: return 0x2611;
				case 794: return 0x2619;
				case 793: return 0x2617;
				case 792: return 0x2609;
				case 791: return 0x260F;
				case 790: return 0x2616;
				case 789: return 0x2614;
				case 788: return 0x2618;
				case 786: return 0x260A;
				case 785: return 0x260E;
				case 784: return 0x2605;
				case 783: return 0x2602;
				case 782: return 0x2603;
				case 781: return 0x2604;
				case 780: return 0x2608;
				case 779: return 0x2607;
				case 778: return 0x261A;
				case 777: return 0x260D;
				case 775: return 0x2613;
				case 776: return 0x2611;
				case 774: return 0x25FB;
				case 773: return 0x25FA;
				case 772: return 0x25F8;
				case 771: return 0x261D;
				case 770: return 0x261C;
				case 769: return 0x25F9;
				case 768: return 0x2612;
				case 766: return 0x25FC;
				case 765: return 0x261E;
				case 764: return 0x261F;
				case 763: return 0x260B;
				case 757: return 0x260B;
				case 756: return 0x260C;
				case 752: return 0x2610;
				case 400: return 0x2106;
				case 401: return 0x2107;
				case 292: return 0x2127;
				case 291: return 0x2126;
				case 290: return 0x2101;
				case 238: return 0x2123;
				case 237: return 0x20D4;
				case 234: return 0x20D4;
				case 233: return 0x20F0;
				case 232: return 0x20EF;
				case 231: return 0x2103;
				case 228: return 0x259E;
				case 226: return 0x2599;
				case 225: return 0x25D3;
				case 223: return 0x20EB;
				case 220: return 0x20F6;
				case 219: return 0x25B4;
				case 218: return 0x25B3;
				case 217: return 0x2588;
				case 216: return 0x20EF;
				case 215: return 0x20D0;
				case 214: return 0x25B5;
				case 213: return 0x20E1;
				case 212: return 0x211E;
				case 211: return 0x2118;
				case 210: return 0x25B2;
				case 209: return 0x2580;
				case 208: return 0x20D1;
				case 207: return 0x20EB;
				case 206: return 0x2131;
				case 205: return 0x2125;
				case 204: return 0x259A;
				case 203: return 0x2101;
				case 202: return 0x2131;
				case 201: return 0x211B;
				case 200: return 0x2598;
				case 195: return 0x260F;
				case 194: return 0x2619;
				case 193: return 0x2615;
				case 192: return 0x25CE;
				case 191: return 0x25A0;
				case 190: return 0x21F1;
				case 189: return 0x25B0;
				case 188: return 0x2615;
				case 187: return 0x2615;
				case 182: return 0x25AE;
				case 181: return 0x25AE;
				case 180: return 0x20D6;
				case 179: return 0x259C;
				case 178: return 0x259C;
				case 177: return 0x259C;
				case 176: return 0x25B6;
				case 175: return 0x2589;
				case 173: return 0x25C3;
				case 172: return 0x20D6;
				case 171: return 0x25B2;
				case 170: return 0x20F6;
				case 167: return 0x2118;
				case 166: return 0x20D7;
				case 165: return 0x2100;
				case 164: return 0x20ED;
				case 163: return 0x25DC;
				case 162: return 0x25DB;
				case 161: return 0x25DA;
				case 160: return 0x25D8;
				case 159: return 0x25D8;
				case 158: return 0x25D6;
				case 157: return 0x25C3;
				case 155: return 0x25B8;
				case 154: return 0x25A7;
				case 153: return 0x2591;
				case 152: return 0x25CA;
				case 151: return 0x20F1;
				case 150: return 0x20FB;
				case 148: return 0x25BE;
				case 147: return 0x25BD;
				case 146: return 0x25BB;
				case 145: return 0x20FB;
				case 144: return 0x25BA;
				case 142: return 0x25B7;
				case 140: return 0x25B1;
				case 139: return 0x25AF;
				case 138: return 0x25AF;
				case 137: return 0x25AA;
				case 136: return 0x25A9;
				case 135: return 0x25A8;
				case 134: return 0x25A1;
				case 133: return 0x2131;
				case 132: return 0x25A0;
				case 131: return 0x2590;
				case 130: return 0x258D;
				case 129: return 0x25C8;
				case 128: return 0x25B6;
				case 127: return 0x2597;
				case 126: return 0x258B;
				case 125: return 0x258B;
				case 124: return 0x258A;
				case 123: return 0x2589;
				case 122: return 0x25CE;
				case 121: return 0x259E;
				case 120: return 0x259E;
				case 119: return 0x259E;
				case 118: return 0x259E;
				case 117: return 0x259D;
				case 116: return 0x259C;
				case 115: return 0x259B;
				case 114: return 0x259B;
				case 113: return 0x20D7;
				case 112: return 0x20D7;
				case 111: return 0x20D7;
				case 110: return 0x20D7;
				case 109: return 0x20D7;
				case 108: return 0x20D7;
				case 107: return 0x20D7;
				case 106: return 0x20D6;
				case 105: return 0x20D6;
				case 104: return 0x20D6;
				case 103: return 0x20D6;
				case 102: return 0x2584;
				case 101: return 0x2581;
				case 100: return 0x25D2;
				case  99: return 0x25D1;
				case  96: return 0x20E8;
				case  98: return 0x2596;
				case  97: return 0x2596;
				case  94: return 0x20E8;
				case  93: return 0x25C2;
				case  92: return 0x25C2;
				case  91: return 0x25C2;
				case  90: return 0x25C1;
				case  89: return 0x25C0;
				case  88: return 0x2108;
				case  87: return 0x25AC;
				case  86: return 0x25AD;
				case  85: return 0x25AB;
				case  84: return 0x25A8;
				case  83: return 0x25A8;
				case  82: return 0x25A5;
				case  81: return 0x2130;
				case  80: return 0x258C;
				case  79: return 0x25A5;
				case  78: return 0x25A5;
				case  77: return 0x25A2;
				case  76: return 0x25CD;
				case  75: return 0x212D;
				case  74: return 0x259F;
				case  73: return 0x2594;
				case  72: return 0x25CB;
				case  71: return 0x25C9;
				case  70: return 0x25CC;
				case  69: return 0x258F;
				case  68: return 0x258F;
				case  67: return 0x258E;
				case  66: return 0x25C8;
				case  65: return 0x25A3;
				case  64: return 0x25A3;
				case  63: return 0x2583;
				case  62: return 0x20D6;
				case  61: return 0x20D6;
				case  60: return 0x20D6;
				case  59: return 0x20D6;
				case  58: return 0x2100;
				case  57: return 0x25BC;
				case  56: return 0x25BC;
				case  55: return 0x20E9;
				case  54: return 0x20E9;
				case  53: return 0x20E9;
				case  52: return 0x25C2;
				case  51: return 0x20E8;
				case  50: return 0x25BC;
				case  49: return 0x20D6;
				case  48: return 0x25B9;
				case  47: return 0x20FA;
				case  46: return 0x20D6;
				case  45: return 0x25B7;
				case  44: return 0x25B7;
				case  43: return 0x2587;
				case  42: return 0x25B7;
				case  41: return 0x25AE;
				case  40: return 0x2586;
				case  39: return 0x25A6;
				case  38: return 0x2585;
				case  37: return 0x25D2;
				case  36: return 0x20DE;
				case  35: return 0x20DE;
				case  34: return 0x25D2;
				case  33: return 0x20DE;
				case  31: return 0x2595;
				case  30: return 0x20DC;
				case  29: return 0x2592;
				case  28: return 0x25C6;
				case  27: return 0x25D1;
				case  26: return 0x25C7;
				case  25: return 0x25D1;
				case  24: return 0x25A4;
				case  23: return 0x25D0;
				case  22: return 0x258F;
				case  21: return 0x25BF;
				case  20: return 0x25C5;
				case  19: return 0x25C4;
				case  18: return 0x20D8;
				case  17: return 0x25AE;
				case  16: return 0x25DD;
				case  15: return 0x25D9;
				case  14: return 0x20D7;
				case  13: return 0x25D7;
				case  12: return 0x20D6;
				case  11: return 0x25C4;
				case  10: return 0x2584;
				case   9: return 0x2584;
				case   8: return 0x2582;
				case   7: return 0x25AF;
				case   6: return 0x211A;
				case   5: return 0x211D;
				case   4: return 0x20D9;
				case   3: return 0x20EC;
				case   2: return 0x20D8;
				case   1: return 0x20DF;

				default:  return 0x1870;


				// TODO: These:
#if false
805	0	781	FFFFFFFF	0	1.3	1.3	1.3	black_ant_worker
806	0	782	FFFFFFFF	0	1.3	1.3	1.3	black_ant_warrior
807	0	783	FFFFFFFF	0	1.3	1.3	1.3	black_ant_queen
808	0	783	FFFFFFFF	0	1.5	1.5	1.5	black_ant_matriarch
804	0	783	FFFFFFFF	0	1.5	1.5	1.5	fire_ant_matriarch
753	0	4	FFFFFFFF	0	1.5	1.5	1.5	gargoyles_gargoyle_slave
754	0	4	FFFFFFFF	0	1.7	1.7	1.7	gargoyles_gargoyle_enforcer
755	0	4	FFFFFFFF	0	2.3	2.3	2.3	gargoyles_gargoyle_guard
758	0	4	FFFFFFFF	0	1.5	1.5	1.5	gargoyles_gargoyle_shopkeeper
168	0	74	FFFFFFFF	0	1.3	1.3	1.3	imps_shadow_fiend
797	0	0	FFFFFFFF	0	1.3	1.3	1.3	mf_dragon_fire
798	0	797	FFFFFFFF	0	1.3	1.3	1.3	mf_dragon_rust
767 	0	769	FFFFFFFF	0	1.55	1.45	1.55	NPC_Blackthorn_Cohort
141	0	0	FFFFFFFF	0	1.3	1.3	1.3	paladins_paladin
149	0	0	FFFFFFFF	0	1.3	1.3	1.3	succubi_succubus
174	0	0	FFFFFFFF	0	2.3	2.3	2.3	succubi_succubus
221	2	0	FFFFFFFF	0	1.3	1.3	1.3	walrus_walrus_male
#endif
			}
		}

		private ArrayList m_List;

		private TrackWhoGump( Mobile from, ArrayList list, int range ) : base( 20, 30 )
		{
			m_From = from;
			m_List = list;
			m_Range = range;

			AddPage( 0 );

			AddBackground( 0, 0, 440, 155, 5054 );

			AddBackground( 10, 10, 420, 75, 2620 );
			AddBackground( 10, 85, 420, 45, 3000 );

			if ( list.Count > 4 )
			{
				AddBackground( 0, 155, 440, 155, 5054 );

				AddBackground( 10, 165, 420, 75, 2620 );
				AddBackground( 10, 240, 420, 45, 3000 );

				if ( list.Count > 8 )
				{
					AddBackground( 0, 310, 440, 155, 5054 );

					AddBackground( 10, 320, 420, 75, 2620 );
					AddBackground( 10, 395, 420, 45, 3000 );
				}
			}

			for ( int i = 0; i < list.Count && i < 12; ++i )
			{
				Mobile m = (Mobile)list[i];

				AddItem( 20 + ((i % 4) * 100), 20 + ((i / 4) * 155), GetItemID( m ) );
				AddButton( 20 + ((i % 4) * 100), 130 + ((i / 4) * 155), 4005, 4007, i + 1, GumpButtonType.Reply, 0 );

				if ( m.Name != null )
					AddHtml( 20 + ((i % 4) * 100), 90 + ((i / 4) * 155), 90, 40, m.Name, false, false );
			}
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			int index = info.ButtonID - 1;

			if ( index >= 0 && index < m_List.Count && index < 12 )
			{
				Mobile m = (Mobile)m_List[index];

				m_From.QuestArrow = new TrackArrow( m_From, m, m_Range * 2 );
			}
		}
	}

	public class TrackArrow : QuestArrow
	{
		private Mobile m_From;
		private Timer m_Timer;

		public TrackArrow( Mobile from, Mobile target, int range ) : base( from )
		{
			m_From = from;
			m_Timer = new TrackTimer( from, target, range, this );
			m_Timer.Start();
		}

		public override void OnClick( bool rightClick )
		{
			if ( rightClick )
			{
				m_From = null;

				Stop();
			}
		}

		public override void OnStop()
		{
			m_Timer.Stop();

			if ( m_From != null )
				m_From.SendLocalizedMessage( 503177 ); // You have lost your quarry.
		}
	}

	public class TrackTimer : Timer
	{
		private Mobile m_From, m_Target;
		private int m_Range;
		private int m_LastX, m_LastY;
		private QuestArrow m_Arrow;

		public TrackTimer( Mobile from, Mobile target, int range, QuestArrow arrow ) : base( TimeSpan.FromSeconds( 0.25 ), TimeSpan.FromSeconds( 2.5 ) )
		{
			m_From = from;
			m_Target = target;
			m_Range = range;

			m_Arrow = arrow;
		}

		protected override void OnTick()
		{
			if ( !m_Arrow.Running )
			{
				Stop();
				return;
			}
			else if ( m_From.NetState == null || m_From.Deleted || m_Target.Deleted || m_From.Map != m_Target.Map || !m_From.InRange( m_Target, m_Range ) )
			{
				m_From.Send( new CancelArrow() );
				m_From.SendLocalizedMessage( 503177 ); // You have lost your quarry.

				Stop();
				return;
			}

			if ( m_LastX != m_Target.X || m_LastY != m_Target.Y )
			{
				m_LastX = m_Target.X;
				m_LastY = m_Target.Y;

				m_Arrow.Update( m_LastX, m_LastY );
			}
		}
	}
}